/*------------------------------------------------------------------------------------------
 																						
  Project:			Alpha Blending

  Module:			query.cpp - Code for querying the display settings from the user.

  Author:			Matthias Hofmann											
																						
--------------------------------------------------------------------------------------------*/


#include "master.h"


// Used for saving the addresses of the 'start' and the 'stop' buttons default
// window procedures. Only used in this module, therefore declared static.
static WNDPROC OldStartProc, OldStopProc;


/*
 * Description:		Queries the display setting from the user.
 *
 * Parameters:		hWnd - Handle to the application window.
 *					lpsi - A pointer to a SSettingsInfo structure.
 *
 * Return value:	If the user cancels, the return value is -1, otherwise 0.
 *
 */
int QuerySettings( HWND hWnd, LPSETTINGSINFO lpsi )
{
	HINSTANCE	hInstance = ( HINSTANCE ) GetWindowLong( hWnd, GWL_HINSTANCE );

	return DialogBoxParam( hInstance, MAKEINTRESOURCE( IDD_QUERY ), hWnd, QueryProc, ( LPARAM ) lpsi );
}

/*
 * Description:		Window procedure for the dialog box.
 *
 * Parameters:		As usual for window procs.
 *
 * Return value:	When DialogBoxParam() returns, a return value of 0 shows that
 *					the user pressed start, 1 indicates that the user cancelled.
 *
 */
BOOL CALLBACK QueryProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	static LPSETTINGSINFO		lpsi;
	LPDIRECTDRAWENUMERATEEX		EnumEx;
	HMODULE						hModule;
	int							iIndex;
	LPGUID						lpGuid;
	int							i;


	switch ( msg )
	{
		case WM_INITDIALOG:
			// Convert the passed parameter.
			lpsi = ( LPSETTINGSINFO ) lParam;

			//
			// Find out wether multimonitoring is supported
			// 
			hModule = LoadLibrary( "ddraw.dll");
			EnumEx = ( LPDIRECTDRAWENUMERATEEX ) GetProcAddress( hModule, "DirectDrawEnumerateExA" );

			// If multimonitoring is supported, call the "Ex" function ...
			if ( EnumEx )
			{
				EnumEx(	EnumDevsEx, hWnd, DDENUM_ATTACHEDSECONDARYDEVICES );
			}
			// ... otherwise, use the old-fashioned one.
			else
			{
				DirectDrawEnumerate( EnumDevs, hWnd );
			}

			// Select the first item in the list
			SendDlgItemMessage( hWnd, IDC_DEVICE, CB_SETCURSEL, 0, 0L ); 

			// Pretend Windows did it so we can respond by saving 
            // the current GUID and updating the list of buttons
            PostMessage( hWnd, WM_COMMAND, MAKELONG( IDC_DEVICE, LBN_SELCHANGE ), 0 );

			//
			// Set new window procedures for the start and the stop button.
			//
			OldStartProc = ( WNDPROC ) SetWindowLong( GetDlgItem( hWnd, IDC_START ), GWL_WNDPROC, ( long ) StartProc );
			OldStopProc = ( WNDPROC ) SetWindowLong( GetDlgItem( hWnd, IDC_STOP ), GWL_WNDPROC, ( long ) StartProc );
				 
			// Set the focus to the start button.
			SetFocus( GetDlgItem( hWnd, IDC_START ) );

			// We dont need the library any more.
			FreeLibrary( hModule );

			// Return 'false' or else the default keyboard focus will be set.
			return false;  

		case WM_COMMAND:
			switch ( LOWORD( wParam ) )
			{
				case IDC_DEVICE:
					if ( HIWORD( wParam ) == LBN_SELCHANGE )
					{
						// Get the current selection
						iIndex = SendDlgItemMessage( hWnd, IDC_DEVICE, CB_GETCURSEL, 0, 0L );

						// Get the pointer to the GUID for the selected device
						lpsi->lpGuid = ( LPGUID ) SendDlgItemMessage( hWnd, IDC_DEVICE, CB_GETITEMDATA, iIndex, 0 );
						
						// Set the radio buttons of the dialog box according to the supported modes
						SetRadioButtons( hWnd, lpsi );	
						
						return true;
					}
					return false;

				case IDC_16BIT:
					lpsi->bDepth = 16;
					return true;

				case IDC_24BIT:
					lpsi->bDepth = 24;
					return true;

				case IDC_32BIT:
					lpsi->bDepth = 32;
					return true;

				case IDC_START:
					// Get the current selection
					iIndex = SendDlgItemMessage( hWnd, IDC_DEVICE, CB_GETCURSEL, 0, 0L );

					//
					// Copy the GUID of the selected device
					//
					lpGuid = ( LPGUID ) SendDlgItemMessage( hWnd, IDC_DEVICE, CB_GETITEMDATA, iIndex, 0 );

					if ( lpGuid )
					{
						lpsi->guid = *lpGuid;
						lpsi->lpGuid = &lpsi->guid;
					}
					else
					{
						lpsi->lpGuid = NULL;
					}

					// Release the memory  previously allocated for the GUIDs
					// ( Could be done via WM_DELETEITEM also, but this wouldnt work on NT < 3.1 )
					iIndex = SendDlgItemMessage( hWnd, IDC_DEVICE, CB_GETCOUNT, 0, 0 );
					
					for ( i = 0; i < iIndex; i++ )
					{
						lpGuid = ( LPGUID ) SendDlgItemMessage( hWnd, IDC_DEVICE, CB_GETITEMDATA, i, 0 );

						if ( lpGuid ) 	
							delete lpGuid;
					}

					// Return that the user pressed start.
					EndDialog( hWnd, 0 );
					return true;

				case IDC_STOP:
					// Return that the user cancelled.
					EndDialog( hWnd, -1 );
					return true;
			}
			return false;
	}
	  
	return false;
}

/*
 * Description:		Callback procedure for DirectDrawEnumerateEx(). If the latter is
 *					not supported, the callback procedure for DirectDrawEnumerate() 
 *					will call this one, with NULL as the HMONITOR parameter.
 *
 * Parameters:		Refer to DirectX SDK documentation.
 *
 * Return value:	DDENUMRET_OK to contimue the enumeration, or DDENUMRET_CANCEL to abort.
 *
 */
BOOL WINAPI EnumDevsEx( LPGUID lpGuid, LPSTR szDesc, LPSTR szName, LPVOID lpContext, HMONITOR hm )
{
	HWND	hWnd =  ( HWND ) lpContext;
	int		iIndex;
	LPVOID	lpDevice;
	

	// Add the display devices name to the list box, retrieving the new entrys index.
	iIndex = SendDlgItemMessage( hWnd, IDC_DEVICE, CB_ADDSTRING, 0, ( LPARAM ) szDesc );

	if ( iIndex != LB_ERR )
	{
		// Copy the GUID. Test wether lpGuid is not NULL, 
		// so we dont dereference a NULL pointer!
		if ( lpGuid )
		{
			lpDevice = new GUID( *lpGuid );	
		}
		else
		{
			lpDevice = NULL;
		}

		// Associate the pointer to the copy with the new entry.
		SendDlgItemMessage( hWnd, IDC_DEVICE, CB_SETITEMDATA, iIndex, ( LPARAM ) lpDevice );
	}
	else
	{
		// If an error ocurs, abort.
		return DDENUMRET_CANCEL;
	}

	// Continue the enumeration.
	return DDENUMRET_OK;
}

/*
 * Description:		Callback procedure for DirectDrawEnumerate(). It calls the 
 *					callback procedure for DirectDrawEnumerateEx() with NULL as
 *					the HMONITOR parameter.
 *
 * Parameters:		Refer to DirectX SDK documentation.
 *
 * Return value:	DDENUMRET_OK to contimue the enumeration, or DDENUMRET_CANCEL to abort.
 *
 */
BOOL WINAPI EnumDevs( LPGUID lpGuid, LPSTR szDesc, LPSTR szName, LPVOID lpContext )
{
	return EnumDevsEx( lpGuid, szDesc, szName, lpContext, NULL );
}

/*
 * Description:		Activates or deactivates the dialog boxs color depth radio buttons
 *					according to the supported modes of the selected display device. 
 *					Every time a display device is selected, it also sets the currently
 *					selected mode to the lowest possible one.
 *
 * Parameters:		hWndDlg - Handle to the dialog box.
 *
 *					lpsi    - Pointer to an SSettingsInfo structure that contains the GUID for
 *					          the currently selected display device. The bDepth member of this
 *							  structure will be set to the new mode.
 *
 * Return value:	At the moment, the function always return 0.
 *
 */
int SetRadioButtons( HWND hWndDlg, LPSETTINGSINFO lpsi )
{
	MODES			modes = { false, false, false };
	LPDIRECTDRAW7	lpDD7;
	DDSURFACEDESC2	ddsd;

	//
	// Create a temporary lpDD7 object
	//
	DirectDrawCreateEx( lpsi->lpGuid, ( void** ) &lpDD7, IID_IDirectDraw7, NULL );

	//
	// Enumerate the display modes
	//	
	ddsd.dwSize = sizeof ddsd;
	ddsd.dwFlags = DDSD_WIDTH | DDSD_HEIGHT;
	ddsd.dwWidth = 800;
	ddsd.dwHeight = 600;

	lpDD7->EnumDisplayModes( 0, &ddsd, &modes, EnumModes ); 

	lpDD7->Release();

	//
	// Enable or disable the radio buttons accordingly
	//
	EnableWindow( GetDlgItem( hWndDlg, IDC_16BIT ), modes.g16Bit );

	EnableWindow( GetDlgItem( hWndDlg, IDC_24BIT ), modes.g24Bit );

	EnableWindow( GetDlgItem( hWndDlg, IDC_32BIT ), modes.g32Bit );

	//
	// Set the lowest possible mode
	//
	if ( modes.g16Bit )
	{
		SendMessage( GetDlgItem( hWndDlg, IDC_16BIT ), BM_SETCHECK, true, 0 );
		SendMessage( GetDlgItem( hWndDlg, IDC_24BIT ), BM_SETCHECK, false, 0 );
		SendMessage( GetDlgItem( hWndDlg, IDC_32BIT ), BM_SETCHECK, false, 0 );
		lpsi->bDepth = 16;
	}
	else if ( modes.g24Bit )
	{
		SendMessage( GetDlgItem( hWndDlg, IDC_16BIT ), BM_SETCHECK, false, 0 );
		SendMessage( GetDlgItem( hWndDlg, IDC_24BIT ), BM_SETCHECK, true, 0 );
		SendMessage( GetDlgItem( hWndDlg, IDC_32BIT ), BM_SETCHECK, false, 0 );
		lpsi->bDepth = 24;
	}
	else if ( modes.g32Bit )
	{
		SendMessage( GetDlgItem( hWndDlg, IDC_16BIT ), BM_SETCHECK, false, 0 );
		SendMessage( GetDlgItem( hWndDlg, IDC_24BIT ), BM_SETCHECK, false, 0 );
		SendMessage( GetDlgItem( hWndDlg, IDC_32BIT ), BM_SETCHECK, true, 0 );
		lpsi->bDepth = 32;
	}

	return 0;
}

/*
 * Description:		Callback procedure for IDirectDraw7::EnumDisplayModes().
 *
 * Parameters:		Refer to DirectX SDK documentation.
 *
 * Return value:	DDENUMRET_OK to contimue the enumeration, or DDENUMRET_CANCEL to abort.
 *
 */
HRESULT WINAPI EnumModes( LPDDSURFACEDESC2 lpddsd, LPVOID lpContext )
{
	LPMODES	lpmodes = ( LPMODES ) lpContext;

	//
	// Set the flags of the passed MODES struct accordingly
	//
	if ( lpddsd->ddpfPixelFormat.dwRGBBitCount == 16 )
		lpmodes->g16Bit = true;

	if ( lpddsd->ddpfPixelFormat.dwRGBBitCount == 24 )
		lpmodes->g24Bit = true;	   

	if ( lpddsd->ddpfPixelFormat.dwRGBBitCount == 32 )
		lpmodes->g32Bit = true;

	// Continue the enumeration.
	return DDENUMRET_OK;
}

/*
 * Description:		Window procedure used for subclassing the start button. 
 *
 * Parameters:		As usual for window procs.
 *
 * Return value:	The value returned from the default window procedure.
 *
 */
LRESULT CALLBACK StartProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch( message )
	{
		// Without responding to this message, subclassing wont work 
		// for the 'return' key ( unless TAB has been pressed before ).
		case WM_GETDLGCODE:
			return DLGC_DEFPUSHBUTTON;

		case WM_KEYDOWN:
			switch ( wParam )
			{  
				// The 'return' key shall be equivalent to pressing the button.
				case VK_RETURN:
					SendMessage( GetParent( hWnd ), WM_COMMAND, MAKELONG( 1, IDC_START ), ( LPARAM ) hWnd );
					break;
			}
			break;
	}

	// Call the former window procedure of this button.
	return CallWindowProc( OldStartProc, hWnd, message, wParam, lParam );
}

/*
 * Description:		Window procedure used for subclassing the stop button. 
 *
 * Parameters:		As usual for window procs.
 *
 * Return value:	The value returned from the default window procedure.
 *
 */
LRESULT CALLBACK StopProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch( message )
	{
		// Without responding to this message, subclassing wont work 
		// for the 'return' key ( unless TAB has been pressed before ).
		case WM_GETDLGCODE:
			return DLGC_DEFPUSHBUTTON;

		case WM_KEYDOWN:
			switch ( wParam )
			{
				// The 'return' key shall be equivalent to pressing the button.
				case VK_RETURN:
					SendMessage( GetParent( hWnd ), WM_COMMAND, MAKELONG( 1, IDC_STOP ), ( LPARAM ) hWnd );
					break;
			}
			break;
	}

	// Call the former window procedure of this button.
	return CallWindowProc( OldStopProc, hWnd, message, wParam, lParam );
}
